@@ -25,7 +25,7 @@ namespace dlib
         !*/
         
     public:
-          virtual const char * what() const throw()
+          virtual const char * what() const noexcept
           {
               return "bad_any_cast";
           }
