@@ -436,6 +436,60 @@ namespace dlib
             - Returns immediately, if size_scale == 1.0
     !*/
 
+// ----------------------------------------------------------------------------------------
+
+    template <
+        typename image_type1,
+        typename image_type2,
+        typename interpolation_type
+        >
+    point_transform_affine letterbox_image (
+        const image_type1& img_in,
+        image_type2& img_out,
+        const interpolation_type interp
+    );
+    /*!
+        requires
+            - image_type1 == an image object that implements the interface defined in
+              dlib/image_processing/generic_image.h
+            - image_type2 == an image object that implements the interface defined in
+              dlib/image_processing/generic_image.h
+            - img_out.size() > 0
+            - interpolation_type == interpolate_nearest_neighbor, interpolate_bilinear,
+              interpolate_quadratic, or a type with a compatible interface.
+            - is_same_object(in_img, out_img) == false
+        ensures
+            - Scales in_img so that it fits into img_out.
+            - Preserves the aspect ratio of in_img by 0-padding the shortest side.
+            - Uses the supplied interpolation routine interp to perform the necessary
+              pixel interpolation.
+            - Returns a transformation object that maps points in in_img into their
+              corresponding location in #out_img.
+    !*/
+
+    template <
+        typename image_type1,
+        typename image_type2
+        >
+    point_transform_affine letterbox_image (
+        const image_type1& img_in,
+        image_type2& img_out
+    );
+    /*!
+        requires
+            - image_type1 == an image object that implements the interface defined in
+              dlib/image_processing/generic_image.h
+            - image_type2 == an image object that implements the interface defined in
+              dlib/image_processing/generic_image.h
+            - img_out.size() > 0
+            - is_same_object(in_img, out_img) == false
+        ensures
+            - Scales in_img so that it fits into img_out using bilinear interpolation.
+            - Preserves the aspect ratio of in_img by 0-padding the shortest side.
+            - Returns a transformation object that maps points in in_img into their
+              corresponding location in #out_img.
+    !*/
+
 // ----------------------------------------------------------------------------------------
 
     template <
@@ -1130,6 +1184,12 @@ namespace dlib
         unsigned long cols;
     };
 
+    void serialize(const chip_details& item, std::ostream& out);
+    void deserialize(chip_details& item, std::istream& in);
+    /*!
+        provides serialization support.
+    !*/
+
 // ----------------------------------------------------------------------------------------
 
     point_transform_affine get_mapping_to_chip (
@@ -1253,6 +1313,50 @@ namespace dlib
               above-defined extract_image_chip() function using bilinear interpolation.
     !*/
 
+// ----------------------------------------------------------------------------------------
+
+    template <
+        typename image_type1,
+        typename image_type2,
+        typename interpolation_type
+    >
+    void insert_image_chip (
+        image_type1& image,
+        const image_type2& chip,
+        const chip_details& location,
+        const interpolation_type& interp
+    );
+    /*!
+        requires
+            - image_type1 == an image object that implements the interface defined in
+              dlib/image_processing/generic_image.h
+            - image_type2 == an image object that implements the interface defined in
+              dlib/image_processing/generic_image.h
+            - pixel_traits<typename image_traits<image_type1>::pixel_type>::has_alpha == false
+            - num_rows(chip) == location.rows && num_columns(chip) == location.cols
+            - interpolation_type == interpolate_nearest_neighbor, interpolate_bilinear, 
+              interpolate_quadratic, or a type with a compatible interface.
+        ensures
+            - This function inserts chip into the image according to the location and the
+              interpolation method supplied as a parameter.
+    !*/
+
+    template <
+        typename image_type1,
+        typename image_type2
+    >
+    void insert_image_chip (
+        image_type1& image,
+        const image_type2& chip,
+        const chip_details& location
+    );
+    /*!
+        ensures
+            - This function is a simple convenience / compatibility wrapper that calls the
+              above-defined insert_image_chip() function using bilinear interpolation.
+    !*/
+
+
 // ----------------------------------------------------------------------------------------
 
     template <
@@ -1453,7 +1557,7 @@ namespace dlib
     template <
         typename image_type
         >
-    void extract_image_4points (
+    point_transform_projective extract_image_4points (
         const image_type& img,
         image_type& out,
         const std::array<dpoint,4>& pts
@@ -1475,12 +1579,14 @@ namespace dlib
               left corner, upper right corner to upper right corner, etc.).
             - #out.nr() == out.nr() && #out.nc() == out.nc().  
               I.e. out should already be sized to whatever size you want it to be.
+            - Returns a transformation object that maps points in img into their
+              corresponding location in #out.
     !*/
 
     template <
         typename image_type
         >
-    void extract_image_4points (
+    point_transform_projective extract_image_4points (
         const image_type& img,
         image_type& out,
         const std::array<line,4>& lines 
@@ -1495,6 +1601,8 @@ namespace dlib
               convex quadrilateral and uses them in a call to the version of
               extract_image_4points() defined above.  i.e. extract_image_4points(img, out,
               intersections_between_lines)
+            - Returns a transformation object that maps points in img into their
+              corresponding location in #out.
         throws 
             - no_convex_quadrilateral: this is thrown if you can't make a convex
               quadrilateral out of the given lines.
